@@ -28,18 +28,6 @@
 #include <sparse/DotIO.h>
 #include <sparse/colorutil.h>
 
-#ifdef _WIN32 /*dependencies*/
-    #pragma comment( lib, "cgraph.lib" )
-    #pragma comment( lib, "gvc.lib" )
-    #pragma comment( lib, "ingraphs.lib" )
-    #pragma comment( lib, "sparse.lib" )
-    #pragma comment( lib, "sfdp.lib" )
-    #pragma comment( lib, "edgepaintlib.lib" )
-    #pragma comment( lib, "neatogen.lib" )
-    #pragma comment( lib, "rbtree.lib" )
-    #pragma comment( lib, "cdt.lib" )
-#endif   /* not _WIN32 */
-
 typedef struct {
   FILE* outfp;
   char** infiles;
@@ -61,23 +49,17 @@ static void usage(char* cmd, int eval)
 {
     fprintf(stderr, "Usage: %s <options> graphfile\n", cmd);
     fputs (usestr, stderr);
-    exit(eval);
+    graphviz_exit(eval);
 }
 
-static FILE *openFile(char *name, char *mode, char* cmd)
+static FILE *openFile(const char *name, const char* cmd)
 {
     FILE *fp;
-    char *modestr;
 
-    fp = fopen(name, mode);
+    fp = fopen(name, "w");
     if (!fp) {
-	if (*mode == 'r')
-	    modestr = "reading";
-	else
-	    modestr = "writing";
-	fprintf(stderr, "%s: could not open file %s for %s\n",
-		cmd, name, modestr);
-	exit(-1);
+	fprintf(stderr, "%s: could not open file %s for writing\n", cmd, name);
+	graphviz_exit(-1);
     }
     return fp;
 }
@@ -107,7 +89,7 @@ static void init(int argc, char *argv[], opts_t* opts) {
       else opts->maxcluster = v;
       break;
     case 'o':
-      opts->outfp = openFile(optarg, "w", cmd);
+      opts->outfp = openFile(optarg, cmd);
       break;
     case 'v':
       Verbose = 1;
@@ -159,5 +141,5 @@ int main(int argc, char *argv[])
     prevg = g;
   }
 
-  return 0; 
+  graphviz_exit(0);
 }
