@@ -51,64 +51,45 @@ static int get_object_type(void)
     return -1;
 }
 
-static void free_attr(attr_t *at) {
-    free(at->defValG);
-    free(at->defValN);
-    free(at->defValE);
-
-    free(at->name);
-    free(at);
-}
-
 static attr_t *new_attr(void) {
-    attr_t *attr = gv_alloc(sizeof(attr_t));
-    attr->defValG = NULL;
-    attr->defValN = NULL;
-    attr->defValE = NULL;
-    attr->name = NULL;
-    attr->value = NULL;
-    attr->propagate = 0;
-    attr->objType[0] = 0;
-    attr->objType[1] = 0;
-    attr->objType[2] = 0;
-    return attr;
+  return gv_alloc(sizeof(attr_t));
 }
 
 static attr_t *new_attr_with_ref(Agsym_t * sym)
 {
-    attr_t *attr = new_attr();
-    attr->name = safestrdup(sym->name);
+    attr_t *a = new_attr();
+    a->name = safestrdup(sym->name);
     switch (sym->kind) {
     case AGRAPH:
-	attr->objType[0] = 1;
+	a->objType[0] = 1;
 	if (sym->defval)
-	    attr->defValG = safestrdup(sym->defval);
+	    a->defValG = safestrdup(sym->defval);
 	break;
     case AGNODE:
-	attr->objType[1] = 1;
+	a->objType[1] = 1;
 	if (sym->defval)
-	    attr->defValN = safestrdup(sym->defval);
+	    a->defValN = safestrdup(sym->defval);
 	break;
     case AGEDGE:
-	attr->objType[2] = 1;
+	a->objType[2] = 1;
 	if (sym->defval)
-	    attr->defValE = safestrdup(sym->defval);
+	    a->defValE = safestrdup(sym->defval);
 	break;
     }
-    return attr;
+    return a;
 }
 
 static attr_t *new_attr_ref(attr_t * refAttr)
 {
-    attr_t *attr = gv_alloc(sizeof(attr_t));
-    *attr = *refAttr;
-    attr->defValG = safestrdup(refAttr->defValG);
-    attr->defValN = safestrdup(refAttr->defValN);
-    attr->defValE = safestrdup(refAttr->defValE);
-    attr->name = safestrdup(refAttr->name);
+    attr_t *a = gv_alloc(sizeof(attr_t));
+    *a = *refAttr;
+    a->defValG = safestrdup(refAttr->defValG);
+    a->defValN = safestrdup(refAttr->defValN);
+    a->defValE = safestrdup(refAttr->defValE);
+    a->name = safestrdup(refAttr->name);
     if (refAttr->value)
-	attr->value = safestrdup(refAttr->value);
-    return attr;
+	a->value = safestrdup(refAttr->value);
+    return a;
 }
 
 static void reset_attr_list_widgets(attr_list * l)
@@ -119,29 +100,11 @@ static void reset_attr_list_widgets(attr_list * l)
     }
 }
 
-static void free_attr_list_widgets(attr_list * l)
-{
-    int id;
-    for (id = 0; id < MAX_FILTERED_ATTR_COUNT; id++) {
-	gtk_object_destroy((GtkObject *) l->fLabels[id]);
-    }
-}
-
-static void free_attr_list(attr_list *l) {
-    int id;
-    for (id = 0; id < l->attr_count; id++) {
-	free_attr(l->attributes[id]);
-    }
-    if (l->with_widgets)
-	free_attr_list_widgets(l);
-    free(l);
-}
-
 // creates a new attr_list
 // attr_list is a basic stack implementation
 // with alphanumeric sorting functions
 // that uses quicksort
-static attr_list *attr_list_new(Agraph_t *g, int with_widgets) {
+static attr_list *attr_list_new(int with_widgets) {
     int id;
     attr_list *l = gv_alloc(sizeof(attr_list));
     l->attr_count = 0;
@@ -328,12 +291,9 @@ static void filter_attributes(const char *prefix, topview *t) {
     int tmp;
 
     attr_list *l = t->attributes;
-    attr_list *fl = t->filtered_attr_list;
     int objKind = get_object_type();
 
-    if (fl)
-	free_attr_list(fl);
-    fl = attr_list_new(NULL, 0);
+    attr_list *fl = attr_list_new(0);
     reset_attr_list_widgets(l);
     create_filtered_list(prefix, l, fl);
     for (ind = 0; ind < fl->attr_count; ind++) {
@@ -416,17 +376,16 @@ static void filter_attributes(const char *prefix, topview *t) {
 
 _BB void on_txtAttr_changed(GtkWidget * widget, gpointer user_data)
 {
+  (void)user_data;
   filter_attributes(gtk_entry_get_text((GtkEntry*)widget), view->Topview);
 }
 
-static void set_refresh_filters(ViewInfo * v, int type, char *name)
+static void set_refresh_filters(ViewInfo * v, char *name)
 {
     if (strcasecmp(name, "pos") == 0)
 	v->refresh.pos = 1;
     if (strcasecmp(name, "color") == 0)
 	v->refresh.color = 1;
-    if ((strcasecmp(name, "size") == 0) && (type == AGNODE))
-	v->refresh.nodesize = 1;
     if (strcasecmp(name, "selected") == 0)
 	v->refresh.selection = 1;
 }
@@ -485,7 +444,7 @@ static void doApply(void)
 	fprintf(stderr,
 	    "on_attrApplyBtn_clicked: unknown object kind %d\n",
 	    objKind);
-    set_refresh_filters(view, objKind, attr_name);
+    set_refresh_filters(view, attr_name);
 }
 
 _BB void on_attrApplyBtn_clicked(GtkWidget * widget, gpointer user_data)
@@ -584,7 +543,7 @@ attr_list *load_attr_list(Agraph_t * g)
     if (!smyrna_attrs)
 	smyrna_attrs = smyrnaPath("attrs.txt");
     g = view->g[view->activeGraph];
-    l = attr_list_new(NULL, 1);
+    l = attr_list_new(1);
     file = fopen(smyrna_attrs, "r");
     if (file != NULL) {
 	int i = 0;
@@ -599,15 +558,15 @@ attr_list *load_attr_list(Agraph_t * g)
 
 		switch (idx) {
 		case 0:
-		    /**/ attr->name = safestrdup(a);
+		    attr->name = safestrdup(a);
 		    break;
 		case 1:
-		    /**/ attr->defValG = safestrdup(a);
+		    attr->defValG = safestrdup(a);
 		    attr->defValN = safestrdup(a);
 		    attr->defValE = safestrdup(a);
 		    break;
 		case 2:
-		    /**/ set_attr_object_type(a, attr->objType);
+		    set_attr_object_type(a, attr->objType);
 		    break;
 		}
 		idx++;
