@@ -8,6 +8,7 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include "gui.h"
@@ -15,8 +16,8 @@
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdk.h>
 #include "viewport.h"
+#include <cgraph/alloc.h>
 #include <cgraph/strview.h>
-#include <common/memory.h>
 
 GladeXML *xml;			//global libglade vars
 GtkWidget *gladewidget;
@@ -31,8 +32,7 @@ void Color_Widget_bg(char *colorstring, GtkWidget * widget)
     gtk_widget_modify_base(widget, GTK_STATE_NORMAL, &color);
 }
 
-void load_graph_properties(Agraph_t * graph)
-{
+void load_graph_properties(void) {
     //dlgOpenGraph , GtkDialog
     gtk_entry_set_text((GtkEntry *)
 		       glade_xml_get_widget(xml, "entryGraphFileName"),
@@ -42,7 +42,7 @@ void load_graph_properties(Agraph_t * graph)
 void load_attributes(void)
 {
     FILE *file;
-    char line[BUFSIZ];
+    char buffer[BUFSIZ];
     char *pch;
     static char *smyrna_attrs;
 
@@ -52,8 +52,9 @@ void load_attributes(void)
     //loads attributes from a text file
     file = fopen(smyrna_attrs, "r");
     if (file != NULL) {
-	for (int attrcount = 0; fgets(line, sizeof line, file) != NULL; ++attrcount) {
-	    pch = strtok(line, ",");
+	for (int attrcount = 0; fgets(buffer, sizeof(buffer), file) != NULL;
+	     ++attrcount) {
+	    pch = strtok(buffer, ",");
 	    for (int ind = 0; pch != NULL; ++ind) {
 		strview_t ss = strview(pch, '\0');
 		pch = strtok(NULL, ",");
@@ -90,9 +91,10 @@ void load_attributes(void)
 		    }
 		    break;
 		default:
-		    attr[attrcount].ComboValues =
-			RALLOC(attr[attrcount].ComboValuesCount,
-			       attr[attrcount].ComboValues, char *);
+		    attr[attrcount].ComboValues = gv_recalloc(attr[attrcount].ComboValues,
+		                                              attr[attrcount].ComboValuesCount - 1,
+		                                              attr[attrcount].ComboValuesCount,
+		                                              sizeof(char*));
 		    attr[attrcount].ComboValues[attr[attrcount].
 						ComboValuesCount] = strview_str(ss);
 		    attr[attrcount].ComboValuesCount++;
@@ -120,17 +122,11 @@ void show_gui_warning(char *str)
 Generic Open File dialog, if a file is selected and return value is 1, else 0
 file name is copied to char* filename,which should be allocated before using the function
 */
-int openfiledlg(int filtercnt, char **filters, agxbuf * xbuf)
-{
+int openfiledlg(char **filename) {
+    assert(filename != NULL);
+
     GtkWidget *dialog;
-    GtkFileFilter *filter;
-    int id, rv;
-    filter = gtk_file_filter_new();
-    if (filtercnt >= 1) {
-	for (id = 0; id < filtercnt; id++) {
-	    gtk_file_filter_add_pattern(filter, filters[id]);
-	}
-    }
+    int rv;
 
     dialog = gtk_file_chooser_dialog_new("Open File",
 					 NULL,
@@ -140,11 +136,8 @@ int openfiledlg(int filtercnt, char **filters, agxbuf * xbuf)
 					 GTK_STOCK_OPEN,
 					 GTK_RESPONSE_ACCEPT, NULL);
 
-    if (filtercnt >= 1)
-	gtk_file_chooser_set_filter((GtkFileChooser *) dialog, filter);
     if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
-	agxbput(xbuf,
-		gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog)));
+	*filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
 	rv = 1;
     } else
 	rv = 0;
@@ -153,17 +146,11 @@ int openfiledlg(int filtercnt, char **filters, agxbuf * xbuf)
     return rv;
 }
 
-int savefiledlg(int filtercnt, char **filters, agxbuf * xbuf)
-{
+int savefiledlg(char **filename) {
+    assert(filename != NULL);
+
     GtkWidget *dialog;
-    GtkFileFilter *filter;
-    int id, rv;
-    filter = gtk_file_filter_new();
-    if (filtercnt >= 1) {
-	for (id = 0; id < filtercnt; id++) {
-	    gtk_file_filter_add_pattern(filter, filters[id]);
-	}
-    }
+    int rv;
 
     dialog = gtk_file_chooser_dialog_new("Save File",
 					 NULL,
@@ -173,11 +160,8 @@ int savefiledlg(int filtercnt, char **filters, agxbuf * xbuf)
 					 GTK_STOCK_OPEN,
 					 GTK_RESPONSE_ACCEPT, NULL);
 
-    if (filtercnt >= 1)
-	gtk_file_chooser_set_filter((GtkFileChooser *) dialog, filter);
     if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
-	agxbput(xbuf,
-		gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog)));
+	*filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
 	rv = 1;
     } else
 	rv = 0;
