@@ -1,3 +1,8 @@
+/**
+ * @file
+ * @brief <a href=https://en.wikipedia.org/wiki/GXL>GXL</a>-DOT convert subroutines
+ */
+
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -11,8 +16,10 @@
 #include    <assert.h>
 #include    "convert.h"
 #include    <cgraph/agxbuf.h>
+#include    <cgraph/alloc.h>
 #include    <cgraph/exit.h>
-#include    <common/memory.h>
+#include    <cgraph/stack.h>
+#include    <stdio.h>
 #ifdef HAVE_EXPAT
 #include    <expat.h>
 #include    <ctype.h>
@@ -24,9 +31,6 @@
 #define XML_STATUS_ERROR 0
 #endif
 
-#define STACK_DEPTH	32
-#define BUFSIZE		20000
-#define SMALLBUF	1000
 #define NAMEBUF		100
 
 #define GXL_ATTR	"_gxl_"
@@ -47,60 +51,10 @@ typedef enum {
   TAG_HTML_LIKE_STRING,
 } attr_t;
 
-typedef struct slist slist;
-struct slist {
-    slist *next;
-    char buf[1];
-};
-
-/* Round x up to next multiple of y, which is a power of 2 */
-#define ROUND2(x,y) (((x) + ((y)-1)) & ~((y)-1))
-
-static void pushString(slist ** stk, const char *s)
-{
-    size_t sz = ROUND2(sizeof(slist) + strlen(s), sizeof(void *));
-    slist *sp = gcalloc(sz, sizeof(char));
-    strcpy(sp->buf, s);
-    sp->next = *stk;
-    *stk = sp;
-}
-
-static void popString(slist ** stk)
-{
-    slist *sp = *stk;
-    if (!sp) {
-	fprintf(stderr, "PANIC: gxl2gv: empty element stack\n");
-	graphviz_exit(1);
-    }
-    *stk = sp->next;
-    free(sp);
-}
-
-static char *topString(slist * stk)
-{
-    if (!stk) {
-	fprintf(stderr, "PANIC: gxl2gv: empty element stack\n");
-	graphviz_exit(1);
-    }
-    return stk->buf;
-}
-
-static void freeString(slist * stk)
-{
-    slist *sp;
-
-    while (stk) {
-	sp = stk->next;
-	free(stk);
-	stk = sp;
-    }
-}
-
 typedef struct {
     agxbuf xml_attr_name;
     agxbuf xml_attr_value;
     agxbuf composite_buffer;
-    slist *elements;
     int listen;
     attr_t closedElementType;
     attr_t globalAttrType;
@@ -115,8 +69,7 @@ static Agraph_t *G;		/* Current graph */
 static Agnode_t *N;		/* Set if Current_class == TAG_NODE */
 static Agedge_t *E;		/* Set if Current_class == TAG_EDGE */
 
-static int GSP;
-static Agraph_t *Gstack[STACK_DEPTH];
+static gv_stack_t Gstack;
 
 typedef struct {
     Dtlink_t link;
@@ -154,32 +107,22 @@ static Dtdisc_t nameDisc = {
     .freef = (Dtfree_f)free_nitem,
 };
 
-static userdata_t *genUserdata(void)
-{
-    userdata_t *user = malloc(sizeof(userdata_t));
-    if (user == NULL)
-	return NULL;
-    agxbinit(&(user->xml_attr_name), NAMEBUF, 0);
-    agxbinit(&(user->xml_attr_value), SMALLBUF, 0);
-    agxbinit(&(user->composite_buffer), SMALLBUF, 0);
-    user->listen = FALSE;
-    user->elements = 0;
-    user->closedElementType = TAG_NONE;
-    user->globalAttrType = TAG_NONE;
-    user->compositeReadState = FALSE;
-    user->edgeinverted = FALSE;
-    user->nameMap = dtopen(&nameDisc, Dtoset);
-    return user;
+static userdata_t genUserdata(void) {
+  userdata_t user = {0};
+  user.listen = FALSE;
+  user.closedElementType = TAG_NONE;
+  user.globalAttrType = TAG_NONE;
+  user.compositeReadState = FALSE;
+  user.edgeinverted = FALSE;
+  user.nameMap = dtopen(&nameDisc, Dtoset);
+  return user;
 }
 
-static void freeUserdata(userdata_t * ud)
-{
-    dtclose(ud->nameMap);
-    agxbfree(&(ud->xml_attr_name));
-    agxbfree(&(ud->xml_attr_value));
-    agxbfree(&(ud->composite_buffer));
-    freeString(ud->elements);
-    free(ud);
+static void freeUserdata(userdata_t ud) {
+  dtclose(ud.nameMap);
+  agxbfree(&ud.xml_attr_name);
+  agxbfree(&ud.xml_attr_value);
+  agxbfree(&ud.composite_buffer);
 }
 
 static void addToMap(Dt_t * map, char *name, char *uniqueName)
@@ -190,7 +133,7 @@ static void addToMap(Dt_t * map, char *name, char *uniqueName)
     obj.name = name;
     objp = dtinsert(map, &obj);
     assert(objp->unique_name == 0);
-    objp->unique_name = strdup(uniqueName);
+    objp->unique_name = gv_strdup(uniqueName);
 }
 
 static char *mapLookup(Dt_t *nm, const char *name) {
@@ -212,26 +155,34 @@ static int isAnonGraph(const char *name)
 
 static void push_subg(Agraph_t * g)
 {
-    if (GSP == STACK_DEPTH) {
-	fprintf(stderr, "gxl2gv: Too many (> %d) nestings of subgraphs\n",
-		STACK_DEPTH);
-	graphviz_exit(1);
-    } else if (GSP == 0)
-	root = g;
-    G = Gstack[GSP++] = g;
+  // insert the new graph
+  stack_push(&Gstack, g);
+
+  // save the root if this is the first graph
+  if (stack_size(&Gstack) == 1) {
+    root = g;
+  }
+
+  // update the top graph
+  G = g;
 }
 
 static Agraph_t *pop_subg(void)
 {
-    Agraph_t *g;
-    if (GSP == 0) {
-	fprintf(stderr, "gxl2gv: Gstack underflow in graph parser\n");
-	graphviz_exit(1);
-    }
-    g = Gstack[--GSP];
-    if (GSP > 0)
-	G = Gstack[GSP - 1];
-    return g;
+  // is the stack empty?
+  if (stack_is_empty(&Gstack)) {
+    fprintf(stderr, "gxl2gv: Gstack underflow in graph parser\n");
+    graphviz_exit(EXIT_FAILURE);
+  }
+
+  // pop the top graph
+  Agraph_t *g = stack_pop(&Gstack);
+
+  // update the top graph
+  if (!stack_is_empty(&Gstack))
+    G = stack_top(&Gstack);
+
+  return g;
 }
 
 static Agnode_t *bind_node(const char *name)
@@ -430,7 +381,6 @@ startElementHandler(void *userData, const char *name, const char **atts)
 	/* do nothing */
     } else if (strcmp(name, "graph") == 0) {
 	const char *edgeMode = "";
-	const char *id;
 	char buf[NAMEBUF];	/* holds % + number */
 
 	Current_class = TAG_GRAPH;
@@ -438,13 +388,18 @@ startElementHandler(void *userData, const char *name, const char **atts)
 	    fprintf(stderr,
 		    "Warning: Node contains more than one graph.\n");
 	}
-	id = atts[get_xml_attr("id", atts)];
+	pos = get_xml_attr("id", atts);
+	if (pos <= 0) {
+	    fprintf(stderr, "Error: Graph has no ID attribute.\n");
+	    graphviz_exit(EXIT_FAILURE);
+	}
+	const char *id = atts[pos];
 	pos = get_xml_attr("edgemode", atts);
 	if (pos > 0) {
 	    edgeMode = atts[pos];
 	}
 
-	if (GSP == 0) {
+	if (stack_is_empty(&Gstack)) {
 	    if (strcmp(edgeMode, "directed") == 0) {
 		g = agopen((char *) id, Agdirected, &AgDefaultDisc);
 	    } else if (strcmp(edgeMode, "undirected") == 0) {
@@ -477,7 +432,6 @@ startElementHandler(void *userData, const char *name, const char **atts)
 	    setGraphAttr(G, GXL_HYPER, (char *) atts[pos], ud);
 	}
 
-	pushString(&ud->elements, id);
     } else if (strcmp(name, "node") == 0) {
 	Current_class = TAG_NODE;
 	pos = get_xml_attr("id", atts);
@@ -485,9 +439,9 @@ startElementHandler(void *userData, const char *name, const char **atts)
 	    const char *attrname;
 	    attrname = atts[pos];
 
-	    bind_node(attrname);
-
-	    pushString(&ud->elements, attrname);
+	    if (attrname != NULL && strcmp(attrname, "") != 0) {
+		bind_node(attrname);
+	    }
 	}
 
     } else if (strcmp(name, "edge") == 0) {
@@ -597,15 +551,8 @@ static void endElementHandler(void *userData, const char *name)
 
     if (strcmp(name, "graph") == 0) {
 	pop_subg();
-	popString(&ud->elements);
 	ud->closedElementType = TAG_GRAPH;
     } else if (strcmp(name, "node") == 0) {
-	char *ele_name = topString(ud->elements);
-	if (ud->closedElementType == TAG_GRAPH) {
-	    Agnode_t *node = agnode(root, ele_name, 0);
-	    agdelete(root, node);
-	}
-	popString(&ud->elements);
 	Current_class = TAG_GRAPH;
 	N = 0;
 	ud->closedElementType = TAG_NODE;
@@ -615,48 +562,38 @@ static void endElementHandler(void *userData, const char *name)
 	ud->closedElementType = TAG_EDGE;
 	ud->edgeinverted = FALSE;
     } else if (strcmp(name, "attr") == 0) {
-	char *new_name;
+	agxbuf new_name = {0};
 	char *value;
-	char buf[SMALLBUF] = GXL_COMP;
-	char *dynbuf = 0;
 
 	ud->closedElementType = TAG_NONE;
 	if (ud->compositeReadState) {
-	    size_t len = sizeof(GXL_COMP) + (size_t)agxblen(&ud->xml_attr_name);
-	    if (len <= SMALLBUF) {
-		new_name = buf;
-	    } else {
-		new_name = dynbuf = gcalloc(len, sizeof(char));
-		strcpy(new_name, GXL_COMP);
-	    }
-	    strcpy(new_name + sizeof(GXL_COMP) - 1,
-		   agxbuse(&ud->xml_attr_name));
+	    agxbprint(&new_name, "%s%s", GXL_COMP, agxbuse(&ud->xml_attr_name));
 	    value = agxbuse(&ud->composite_buffer);
 	    agxbclear(&ud->xml_attr_value);
 	    ud->compositeReadState = FALSE;
 	} else {
-	    new_name = agxbuse(&ud->xml_attr_name);
+	    agxbput(&new_name, agxbuse(&ud->xml_attr_name));
 	    value = agxbuse(&ud->xml_attr_value);
 	}
 
 	switch (ud->globalAttrType) {
 	case TAG_NONE:
-	    setAttr(new_name, value, ud, false);
+	    setAttr(agxbuse(&new_name), value, ud, false);
 	    break;
 	case TAG_NODE:
-	    setGlobalNodeAttr(G, new_name, value);
+	    setGlobalNodeAttr(G, agxbuse(&new_name), value);
 	    break;
 	case TAG_EDGE:
-	    setGlobalEdgeAttr(G, new_name, value);
+	    setGlobalEdgeAttr(G, agxbuse(&new_name), value);
 	    break;
 	case TAG_GRAPH:
-	    setGraphAttr(G, new_name, value, ud);
+	    setGraphAttr(G, agxbuse(&new_name), value, ud);
 	    break;
 	case TAG_HTML_LIKE_STRING:
-	    setAttr(new_name, value, ud, true);
+	    setAttr(agxbuse(&new_name), value, ud, true);
 	    break;
 	}
-	free(dynbuf);
+	agxbfree(&new_name);
 	ud->globalAttrType = TAG_NONE;
     } else if (strcmp(name, "string") == 0
 	       || strcmp(name, "bool") == 0
@@ -693,17 +630,12 @@ static void characterDataHandler(void *userData, const char *s, int length)
 
 Agraph_t *gxl_to_gv(FILE * gxlFile)
 {
-    char buf[BUFSIZE];
+    char buf[BUFSIZ];
     int done;
-    userdata_t *udata = genUserdata();
+    userdata_t udata = genUserdata();
     XML_Parser parser = XML_ParserCreate(NULL);
 
-    if (udata == NULL) {
-	fprintf(stderr, "out of memory\n");
-	graphviz_exit(1);
-    }
-
-    XML_SetUserData(parser, udata);
+    XML_SetUserData(parser, &udata);
     XML_SetElementHandler(parser, startElementHandler, endElementHandler);
     XML_SetCharacterDataHandler(parser, characterDataHandler);
 
@@ -725,6 +657,7 @@ Agraph_t *gxl_to_gv(FILE * gxlFile)
     } while (!done);
     XML_ParserFree(parser);
     freeUserdata(udata);
+    stack_reset(&Gstack);
 
     return root;
 }
