@@ -8,10 +8,11 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-
+#include	<cgraph/list.h>
+#include	<cgraph/agxbuf.h>
+#include	<cgraph/alloc.h>
 #include	<circogen/blockpath.h>
 #include	<circogen/edgelist.h>
-#include	<circogen/deglist.h>
 #include	<stddef.h>
 #include	<stdbool.h>
 
@@ -35,14 +36,15 @@ static Agraph_t *clone_graph(Agraph_t * ing, Agraph_t ** xg)
     Agnode_t *xh;
     Agedge_t *e;
     Agedge_t *xe;
-    char gname[SMALLBUF];
+    agxbuf gname = {0};
     static int id = 0;
 
-    snprintf(gname, sizeof(gname), "_clone_%d", id++);
-    clone = agsubg(ing, gname,1);
+    agxbprint(&gname, "_clone_%d", id++);
+    clone = agsubg(ing, agxbuse(&gname), 1);
     agbindrec(clone, "Agraphinfo_t", sizeof(Agraphinfo_t), true);	//node custom data
-    snprintf(gname, sizeof(gname), "_clone_%d", id++);
-    xclone = agopen(gname, ing->desc,NULL);
+    agxbprint(&gname, "_clone_%d", id++);
+    xclone = agopen(agxbuse(&gname), ing->desc, NULL);
+    agxbfree(&gname);
     for (n = agfstnode(ing); n; n = agnxtnode(ing, n)) {
 	agsubnode(clone,n,1);
 	xn = agnode(xclone, agnameof(n),1);
@@ -66,17 +68,39 @@ static Agraph_t *clone_graph(Agraph_t * ing, Agraph_t ** xg)
     return clone;
 }
 
+DEFINE_LIST(deglist, Agnode_t*)
+
+/// comparison function for sorting nodes by degree, descending
+static int cmpDegree(const Agnode_t **a, const Agnode_t **b) {
+// Suppress Clang/GCC -Wcast-qual warnings. `DEGREE` does not modify the
+// underlying data, but uses casts that make it look like it does.
+#ifdef __GNUC__
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-qual"
+#endif
+  if (DEGREE(*a) < DEGREE(*b)) {
+    return 1;
+  }
+  if (DEGREE(*a) > DEGREE(*b)) {
+    return -1;
+  }
+#ifdef __GNUC__
+#pragma GCC diagnostic pop
+#endif
+  return 0;
+}
+
 /* fillList:
- * Add nodes to deg_list, which stores them by degree.
+ * Add nodes to deg_list, storing them by descending degree.
  */
-static deglist_t *getList(Agraph_t * g)
-{
-    deglist_t *dl = mkDeglist();
+static deglist_t getList(Agraph_t *g) {
+    deglist_t dl = {0};
     Agnode_t *n;
 
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	insertDeglist(dl, n);
+	deglist_append(&dl, n);
     }
+    deglist_sort(&dl, cmpDegree);
     return dl;
 }
 
@@ -84,9 +108,6 @@ static deglist_t *getList(Agraph_t * g)
  */
 static void find_pair_edges(Agraph_t * g, Agnode_t * n, Agraph_t * outg)
 {
-    Agnode_t **neighbors_with;
-    Agnode_t **neighbors_without;
-
     Agedge_t *e;
     Agedge_t *ep;
     Agedge_t *ex;
@@ -100,8 +121,8 @@ static void find_pair_edges(Agraph_t * g, Agnode_t * n, Agraph_t * outg)
     int edge_cnt = 0;
 
     node_degree = DEGREE(n);
-    neighbors_with = N_GNEW(node_degree, Agnode_t *);
-    neighbors_without = N_GNEW(node_degree, Agnode_t *);
+    Agnode_t **neighbors_with = gv_calloc(node_degree, sizeof(Agnode_t*));
+    Agnode_t **neighbors_without = gv_calloc(node_degree, sizeof(Agnode_t*));
 
     for (e = agfstedge(g, n); e; e = agnxtedge(g, e, n)) {
 	n1 = aghead(e);
@@ -190,23 +211,22 @@ static Agraph_t *remove_pair_edges(Agraph_t * ing)
     int nodeCount;
     Agraph_t *outg;
     Agraph_t *g;
-    deglist_t *dl;
     Agnode_t *currnode, *adjNode;
     Agedge_t *e;
 
     outg = clone_graph(ing, &g);
     nodeCount = agnnodes(g);
-    dl = getList(g);
+    deglist_t dl = getList(g);
 
-    while (counter < (nodeCount - 3)) {
-	currnode = firstDeglist(dl);
+    while (counter < nodeCount - 3) {
+	currnode = deglist_is_empty(&dl) ? NULL : deglist_pop(&dl);
 
 	/* Remove all adjacent nodes since they have to be reinserted */
 	for (e = agfstedge(g, currnode); e; e = agnxtedge(g, e, currnode)) {
 	    adjNode = aghead(e);
 	    if (currnode == adjNode)
 		adjNode = agtail(e);
-	    removeDeglist(dl, adjNode);
+	    deglist_remove(&dl, adjNode);
 	}
 
 	find_pair_edges(g, currnode, outg);
@@ -217,8 +237,9 @@ static Agraph_t *remove_pair_edges(Agraph_t * ing)
 		adjNode = agtail(e);
 
 	    DEGREE(adjNode)--;
-	    insertDeglist(dl, adjNode);
+	    deglist_append(&dl, adjNode);
 	}
+	deglist_sort(&dl, cmpDegree);
 
 	agdelete(g, currnode);
 
@@ -226,7 +247,7 @@ static Agraph_t *remove_pair_edges(Agraph_t * ing)
     }
 
     agclose(g);
-    freeDeglist(dl);
+    deglist_free(&dl);
     return outg;
 }
 
@@ -251,7 +272,7 @@ measure_distance(Agnode_t * n, Agnode_t * ancestor, int dist,
 	DISTONE(parent) = dist;
     } else if (dist > DISTONE(parent)) {
 	if (LEAFONE(parent) != change) {
-	    if (!DISTTWO(parent) || (LEAFTWO(parent) != change))
+	    if (!DISTTWO(parent) || LEAFTWO(parent) != change)
 		change = LEAFONE(parent);
 	    LEAFTWO(parent) = LEAFONE(parent);
 	    DISTTWO(parent) = DISTONE(parent);
@@ -276,17 +297,16 @@ static nodelist_t *find_longest_path(Agraph_t * tree)
     Agnode_t *n;
     Agedge_t *e;
     Agnode_t *common = 0;
-    nodelist_t *path;
     nodelist_t *endPath;
     int maxlength = 0;
     int length;
 
     if (agnnodes(tree) == 1) {
-	path = mkNodelist();
+	nodelist_t *beginPath = mkNodelist();
 	n = agfstnode(tree);
-	appendNodelist(path, NULL, n);
+	appendNodelist(beginPath, NULL, n);
 	SET_ONPATH(n);
-	return path;
+	return beginPath;
     }
 
     for (n = agfstnode(tree); n; n = agnxtnode(tree, n)) {
@@ -307,12 +327,12 @@ static nodelist_t *find_longest_path(Agraph_t * tree)
 	}
     }
 
-    path = mkNodelist();
+    nodelist_t *beginPath = mkNodelist();
     for (n = LEAFONE(common); n != common; n = TPARENT(n)) {
-	appendNodelist(path, NULL, n);
+	appendNodelist(beginPath, NULL, n);
 	SET_ONPATH(n);
     }
-    appendNodelist(path, NULL, common);
+    appendNodelist(beginPath, NULL, common);
     SET_ONPATH(common);
 
     if (DISTTWO(common)) {	/* 2nd path might be empty */
@@ -321,10 +341,10 @@ static nodelist_t *find_longest_path(Agraph_t * tree)
 	    appendNodelist(endPath, NULL, n);
 	    SET_ONPATH(n);
 	}
-	reverseAppend(path, endPath);
+	reverseAppend(beginPath, endPath);
     }
 
-    return path;
+    return beginPath;
 }
 
 /* dfs:
@@ -357,11 +377,12 @@ static Agraph_t *spanning_tree(Agraph_t * g)
 {
     Agnode_t *n;
     Agraph_t *tree;
-    char gname[SMALLBUF];
+    agxbuf gname = {0};
     static int id = 0;
 
-    snprintf(gname, sizeof(gname), "_span_%d", id++);
-    tree = agsubg(g, gname,1);
+    agxbprint(&gname, "_span_%d", id++);
+    tree = agsubg(g, agxbuse(&gname), 1);
+    agxbfree(&gname);
     agbindrec(tree, "Agraphinfo_t", sizeof(Agraphinfo_t), true);	//node custom data
 
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
@@ -425,7 +446,7 @@ static int count_all_crossings(nodelist_t * list, Agraph_t * subg)
 		     eitem = dtnext(openEdgeList, eitem)) {
 		    ep = eitem->edge;
 		    if (EDGEORDER(ep) > EDGEORDER(e)) {
-			if ((aghead(ep) != n) && (agtail(ep) != n))
+			if (aghead(ep) != n && agtail(ep) != n)
 			    crossings++;
 		    }
 		}
@@ -506,7 +527,7 @@ static nodelist_t *reduce_edge_crossings(nodelist_t * list,
 	origCrossings = crossings;
 	list = reduce(list, subg, &crossings);
 	/* return if no crossings or no improvement */
-	if ((origCrossings == crossings) || (crossings == 0))
+	if (origCrossings == crossings || crossings == 0)
 	    return list;
     }
     return list;
@@ -627,7 +648,7 @@ nodelist_t *layout_block(Agraph_t * g, block_t * sn, double min_dist)
     if (N == 1)
 	radius = 0;
     else
-	radius = (N * (min_dist + largest_node)) / (2 * M_PI);
+	radius = N * (min_dist + largest_node) / (2 * M_PI);
 
     for (item = longest_path->first; item; item = item->next) {
 	n = item->curr;
@@ -643,7 +664,7 @@ nodelist_t *layout_block(Agraph_t * g, block_t * sn, double min_dist)
 	n = item->curr;
 	POSITION(n) = k;
 	PSI(n) = 0.0;
-	theta = k * ((2.0 * M_PI) / N);
+	theta = k * (2.0 * M_PI / N);
 
 	ND_pos(n)[0] = radius * cos(theta);
 	ND_pos(n)[1] = radius * sin(theta);
