@@ -9,14 +9,18 @@
  *************************************************************************/
 
 #include "config.h"
+#include <assert.h>
 #include <cgraph/agxbuf.h>
 #include <cgraph/alloc.h>
+#include <common/geomprocs.h>
 #include <common/render.h>
+#include <limits.h>
 #include <math.h>
 #include <pathplan/pathplan.h>
 #include <setjmp.h>
 #include <stdbool.h>
 #include <stdlib.h>
+#include <string.h>
 
 static int nedges, nboxes; /* total no. of edges and boxes used in routing */
 
@@ -34,17 +38,13 @@ static void printboxes(int boxn, boxf* boxes)
 {
     pointf ll, ur;
     int bi;
-    int newcnt = Show_cnt + boxn;
 
-    Show_boxes = ALLOC(newcnt+2,Show_boxes,char*);
     for (bi = 0; bi < boxn; bi++) {
 	ll = boxes[bi].LL, ur = boxes[bi].UR;
 	agxbuf buf = {0};
 	agxbprint(&buf, "%.0f %.0f %.0f %.0f pathbox", ll.x, ll.y, ur.x, ur.y);
-	Show_boxes[bi + 1 + Show_cnt] = agxbdisown(&buf);
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
     }
-    Show_cnt = newcnt;
-    Show_boxes[Show_cnt+1] = NULL;
 }
 
 #if DEBUG > 1
@@ -85,57 +85,44 @@ static void psprintpointf(pointf p)
 
 static void psprintspline(Ppolyline_t spl)
 {
-    int newcnt = Show_cnt + spl.pn + 4;
-    int li, i;
+    int i;
 
-    Show_boxes = ALLOC(newcnt+2,Show_boxes,char*);
-    li = Show_cnt+1;
-    Show_boxes[li++] = gv_strdup("%%!");
-    Show_boxes[li++] = gv_strdup("%% spline");
-    Show_boxes[li++] = gv_strdup("gsave 1 0 0 setrgbcolor newpath");
+    show_boxes_append(&Show_boxes, gv_strdup("%%!"));
+    show_boxes_append(&Show_boxes, gv_strdup("%% spline"));
+    show_boxes_append(&Show_boxes, gv_strdup("gsave 1 0 0 setrgbcolor newpath"));
     for (i = 0; i < spl.pn; i++) {
 	agxbuf buf = {0};
 	agxbprint(&buf, "%f %f %s", spl.ps[i].x, spl.ps[i].y,
 	  i == 0 ?  "moveto" : (i % 3 == 0 ? "curveto" : ""));
-	Show_boxes[li++] = agxbdisown(&buf);
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
     }
-    Show_boxes[li++] = gv_strdup("stroke grestore");
-    Show_cnt = newcnt;
-    Show_boxes[Show_cnt+1] = NULL;
+    show_boxes_append(&Show_boxes, gv_strdup("stroke grestore"));
 }
 
 static void psprintline(Ppolyline_t pl)
 {
-    int newcnt = Show_cnt + pl.pn + 4;
-    int i, li;
+    int i;
 
-    Show_boxes = ALLOC(newcnt+2,Show_boxes,char*);
-    li = Show_cnt+1;
-    Show_boxes[li++] = gv_strdup("%%!");
-    Show_boxes[li++] = gv_strdup("%% line");
-    Show_boxes[li++] = gv_strdup("gsave 0 0 1 setrgbcolor newpath");
+    show_boxes_append(&Show_boxes, gv_strdup("%%!"));
+    show_boxes_append(&Show_boxes, gv_strdup("%% line"));
+    show_boxes_append(&Show_boxes, gv_strdup("gsave 0 0 1 setrgbcolor newpath"));
     for (i = 0; i < pl.pn; i++) {
 	agxbuf buf = {0};
 	agxbprint(&buf, "%f %f %s", pl.ps[i].x, pl.ps[i].y,
 		i == 0 ? "moveto" : "lineto");
-	Show_boxes[li++] = agxbdisown(&buf);
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
     }
-    Show_boxes[li++] = gv_strdup("stroke grestore");
-    Show_cnt = newcnt;
-    Show_boxes[Show_cnt+1] = NULL;
+    show_boxes_append(&Show_boxes, gv_strdup("stroke grestore"));
 }
 
 static void psprintpoly(Ppoly_t p)
 {
-    int newcnt = Show_cnt + p.pn + 3;
     point tl, hd;
-    int bi, li;
+    int bi;
     char*  pfx;
 
-    Show_boxes = ALLOC(newcnt+2,Show_boxes,char*);
-    li = Show_cnt+1;
-    Show_boxes[li++] = gv_strdup("%% poly list");
-    Show_boxes[li++] = gv_strdup("gsave 0 1 0 setrgbcolor");
+    show_boxes_append(&Show_boxes, gv_strdup("%% poly list"));
+    show_boxes_append(&Show_boxes, gv_strdup("gsave 0 1 0 setrgbcolor"));
     for (bi = 0; bi < p.pn; bi++) {
 	tl.x = (int)p.ps[bi].x;
 	tl.y = (int)p.ps[bi].y;
@@ -145,54 +132,40 @@ static void psprintpoly(Ppoly_t p)
 	else pfx ="";
 	agxbuf buf = {0};
 	agxbprint(&buf, "%s%d %d %d %d makevec", pfx, tl.x, tl.y, hd.x, hd.y);
-	Show_boxes[li++] = agxbdisown(&buf);
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
     }
-    Show_boxes[li++] = gv_strdup("grestore");
-
-    Show_cnt = newcnt;
-    Show_boxes[Show_cnt+1] = NULL;
+    show_boxes_append(&Show_boxes, gv_strdup("grestore"));
 }
 
 static void psprintboxes(int boxn, boxf* boxes)
 {
-    int newcnt = Show_cnt + 5*boxn + 3;
     pointf ll, ur;
-    int bi, li;
+    int bi;
 
-    Show_boxes = ALLOC(newcnt+2,Show_boxes,char*);
-    li = Show_cnt+1;
-    Show_boxes[li++] = gv_strdup("%% box list");
-    Show_boxes[li++] = gv_strdup("gsave 0 1 0 setrgbcolor");
+    show_boxes_append(&Show_boxes, gv_strdup("%% box list"));
+    show_boxes_append(&Show_boxes, gv_strdup("gsave 0 1 0 setrgbcolor"));
     for (bi = 0; bi < boxn; bi++) {
 	ll = boxes[bi].LL, ur = boxes[bi].UR;
 	agxbuf buf = {0};
 	agxbprint(&buf, "newpath\n%.0f %.0f moveto", ll.x, ll.y);
-	Show_boxes[li++] = agxbdisown(&buf);
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
 	agxbprint(&buf, "%.0f %.0f lineto", ll.x, ur.y);
-	Show_boxes[li++] = agxbdisown(&buf);
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
 	agxbprint(&buf, "%.0f %.0f lineto", ur.x, ur.y);
-	Show_boxes[li++] = agxbdisown(&buf);
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
 	agxbprint(&buf, "%.0f %.0f lineto", ur.x, ll.y);
-	Show_boxes[li++] = agxbdisown(&buf);
-	Show_boxes[li++] = gv_strdup("closepath stroke");
+	show_boxes_append(&Show_boxes, agxbdisown(&buf));
+	show_boxes_append(&Show_boxes, gv_strdup("closepath stroke"));
     }
-    Show_boxes[li++] = gv_strdup("grestore");
-
-    Show_cnt = newcnt;
-    Show_boxes[Show_cnt+1] = NULL;
+    show_boxes_append(&Show_boxes, gv_strdup("grestore"));
 }
 
 static void psprintinit (int begin)
 {
-    int newcnt = Show_cnt + 1;
-
-    Show_boxes = ALLOC(newcnt+2,Show_boxes,char*);
     if (begin)
-	Show_boxes[1+Show_cnt] = gv_strdup("dbgstart");
+	show_boxes_append(&Show_boxes, gv_strdup("dbgstart"));
     else
-	Show_boxes[1+Show_cnt] = gv_strdup("grestore");
-    Show_cnt = newcnt;
-    Show_boxes[Show_cnt+1] = NULL;
+	show_boxes_append(&Show_boxes, gv_strdup("grestore"));
 }
 
 static bool debugleveln(edge_t* realedge, int i)
@@ -264,13 +237,7 @@ routesplinesinit()
 {
     if (++routeinit > 1) return 0;
 #ifdef DEBUG
-    if (Show_boxes) {
-        for (int i = 0; Show_boxes[i]; i++)
-	    free (Show_boxes[i]);
-	free (Show_boxes);
-	Show_boxes = NULL;
-	Show_cnt = 0;
-    }
+    show_boxes_free(&Show_boxes);
 #endif
     nedges = 0;
     nboxes = 0;
@@ -500,8 +467,12 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	    polypoints[i].y *= -1;
     }
 
-    for (bi = 0; bi < boxn; bi++)
-	boxes[bi].LL.x = INT_MAX, boxes[bi].UR.x = INT_MIN;
+    static const double INITIAL_LLX = INT_MAX;
+    static const double INITIAL_URX = INT_MIN;
+    for (bi = 0; bi < boxn; bi++) {
+	boxes[bi].LL.x = INITIAL_LLX;
+	boxes[bi].UR.x = INITIAL_URX;
+    }
     poly.ps = polypoints, poly.pn = pi;
     eps[0].x = pp->start.p.x, eps[0].y = pp->start.p.y;
     eps[1].x = pp->end.p.x, eps[1].y = pp->end.p.y;
@@ -556,10 +527,6 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	return NULL;  /* Bailout if no memory left */
     }
 
-    for (bi = 0; bi < boxn; bi++) {
-	boxes[bi].LL.x = INT_MAX;
-	boxes[bi].UR.x = INT_MIN;
-    }
     unbounded = true;
     for (splinepi = 0; splinepi < spl.pn; splinepi++) {
 	ps[splinepi] = spl.ps[splinepi];
@@ -576,7 +543,8 @@ static pointf *_routesplines(path * pp, int *npoints, int polyline)
 	for (bi = 0; bi < boxn; bi++) {
 	/* these fp equality tests are used only to detect if the
 	 * values have been changed since initialization - ok */
-	    if (boxes[bi].LL.x == INT_MAX || boxes[bi].UR.x == INT_MIN) {
+	    if (memcmp(&boxes[bi].LL.x, &INITIAL_LLX, sizeof(boxes[bi].LL.x)) == 0 ||
+	        memcmp(&boxes[bi].UR.x, &INITIAL_URX, sizeof(boxes[bi].UR.x)) == 0) {
 		delta *= 2; /* try again with a finer interval */
 		if (delta > INT_MAX/boxn) /* in limitBoxes, boxn*delta must fit in an int, so give up */
 		    loopcnt = LOOP_TRIES;
@@ -688,7 +656,7 @@ static int checkpath(int boxn, boxf* boxes, path* thepath)
 	    printpath(thepath);
 	}
 	if (errs > 0) {
-	    int xy;
+	    double xy;
 
 	    if (l == 1)
 		xy = ba->UR.x, ba->UR.x = bb->LL.x, bb->LL.x = xy, l = 0;
@@ -1017,7 +985,6 @@ static pointf get_cycle_centroid(graph_t *g, edge_t* edge)
 	//cycles of length 2 do their own thing, we want 3 or
 	vec* cycle = find_shortest_cycle_with_edge(cycles, edge, 3);
 	size_t cycle_len;
-	size_t cnt = 0;
     pointf sum = {0.0, 0.0};
 	size_t idx; //edge index
 	node_t *n;
@@ -1030,6 +997,7 @@ static pointf get_cycle_centroid(graph_t *g, edge_t* edge)
 
 	cycle_len = vec_length(cycle);
 
+	double cnt = 0;
 	for (idx=0; idx < cycle_len; ++idx) {
 		n = vec_get(cycle, idx);
 		sum.x += ND_coord(n).x;
@@ -1047,15 +1015,11 @@ static pointf get_cycle_centroid(graph_t *g, edge_t* edge)
 
 static void bend(pointf spl[4], pointf centroid)
 {
-    pointf  midpt,a;
+    pointf  a;
     double  r;
-    double  dist,dx,dy;
 
-    midpt.x = (spl[0].x + spl[3].x)/2.0;
-    midpt.y = (spl[0].y + spl[3].y)/2.0;
-    dx = spl[3].x - spl[0].x;
-    dy = spl[3].y - spl[0].y;
-    dist = hypot(dx, dy);
+    pointf midpt = mid_pointf(spl[0], spl[3]);
+    double dist = DIST(spl[3], spl[0]);
     r = dist/5.0;
     {
         double vX = centroid.x - midpt.x;
@@ -1078,50 +1042,42 @@ void
 makeStraightEdge(graph_t * g, edge_t * e, int et, splineInfo* sinfo)
 {
     edge_t *e0;
-    int i, e_cnt;
 
-    e_cnt = 1;
+    size_t e_cnt = 1;
     e0 = e;
     while (e0 != ED_to_virt(e0) && (e0 = ED_to_virt(e0))) e_cnt++;
 
-    edge_t **edges = N_NEW(e_cnt, edge_t*);
+    edge_t **edge_list = N_NEW(e_cnt, edge_t*);
     e0 = e;
-    for (i = 0; i < e_cnt; i++) {
-	edges[i] = e0;
+    for (size_t i = 0; i < e_cnt; i++) {
+	edge_list[i] = e0;
 	e0 = ED_to_virt(e0);
     }
-    makeStraightEdges (g, edges, e_cnt, et, sinfo);
-    free(edges);
+    assert(e_cnt <= INT_MAX);
+    makeStraightEdges(g, edge_list, (int)e_cnt, et, sinfo);
+    free(edge_list);
 }
 
 void 
-makeStraightEdges(graph_t * g, edge_t** edges, int e_cnt, int et, splineInfo* sinfo)
-{
+makeStraightEdges(graph_t *g, edge_t **edge_list, int e_cnt, int et,
+                  splineInfo *sinfo) {
     pointf dumb[4];
-    node_t *n;
-    node_t *head;
     bool curved = et == EDGETYPE_CURVED;
-    pointf perp;
     pointf del;
-    edge_t *e0;
-    edge_t *e;
-    int i, j, xstep, dx;
-    double l_perp;
-    pointf dumber[4];
 
-    e = edges[0];
-    n = agtail(e);
-    head = aghead(e);
+    edge_t *e = edge_list[0];
+    node_t *n = agtail(e);
+    node_t *head = aghead(e);
     dumb[1] = dumb[0] = add_pointf(ND_coord(n), ED_tail_port(e).p);
     dumb[2] = dumb[3] = add_pointf(ND_coord(head), ED_head_port(e).p);
     if (e_cnt == 1 || Concentrate) {
-	if (curved) bend(dumb,get_cycle_centroid(g, edges[0]));
+	if (curved) bend(dumb,get_cycle_centroid(g, edge_list[0]));
 	clip_and_install(e, aghead(e), dumb, 4, sinfo);
 	addEdgeLabels(e);
 	return;
     }
 
-    e0 = e;
+    edge_t *e0 = e;
     if (APPROXEQPT(dumb[0], dumb[3], MILLIPOINT)) {
 	/* degenerate case */
 	dumb[1] = dumb[0];
@@ -1130,11 +1086,13 @@ makeStraightEdges(graph_t * g, edge_t** edges, int e_cnt, int et, splineInfo* si
 	del.y = 0;
     }
     else {
-        perp.x = dumb[0].y - dumb[3].y;
-        perp.y = dumb[3].x - dumb[0].x;
-	l_perp = LEN(perp.x, perp.y);
-	xstep = GD_nodesep(g->root);
-	dx = xstep * (e_cnt - 1) / 2;
+        pointf perp = {
+          .x = dumb[0].y - dumb[3].y,
+          .y = dumb[3].x - dumb[0].x
+        };
+	double l_perp = hypot(perp.x, perp.y);
+	int xstep = GD_nodesep(g->root);
+	int dx = xstep * (e_cnt - 1) / 2;
 	dumb[1].x = dumb[0].x + dx * perp.x / l_perp;
 	dumb[1].y = dumb[0].y + dx * perp.y / l_perp;
 	dumb[2].x = dumb[3].x + dx * perp.x / l_perp;
@@ -1143,26 +1101,21 @@ makeStraightEdges(graph_t * g, edge_t** edges, int e_cnt, int et, splineInfo* si
 	del.y = -xstep * perp.y / l_perp;
     }
 
-    for (i = 0; i < e_cnt; i++) {
-	e0 = edges[i];
+    for (int i = 0; i < e_cnt; i++) {
+	e0 = edge_list[i];
+	pointf dumber[4];
 	if (aghead(e0) == head) {
-	    for (j = 0; j < 4; j++) {
+	    for (size_t j = 0; j < 4; j++) {
 		dumber[j] = dumb[j];
 	    }
 	} else {
-	    for (j = 0; j < 4; j++) {
+	    for (size_t j = 0; j < 4; j++) {
 		dumber[3 - j] = dumb[j];
 	    }
 	}
 	if (et == EDGETYPE_PLINE) {
-	    Ppoint_t pts[4];
-	    Ppolyline_t spl, line;
-
-	    line.pn = 4;
-	    line.ps = pts;
-	    for (j=0; j < 4; j++) {
-		pts[j] = dumber[j];
-	    }
+	    Ppoint_t pts[] = {dumber[0], dumber[1], dumber[2], dumber[3]};
+	    Ppolyline_t spl, line = {.pn = 4, .ps = pts};
 	    make_polyline (line, &spl);
 	    clip_and_install(e0, aghead(e0), spl.ps, spl.pn, sinfo);
 	}
@@ -1170,9 +1123,7 @@ makeStraightEdges(graph_t * g, edge_t** edges, int e_cnt, int et, splineInfo* si
 	    clip_and_install(e0, aghead(e0), dumber, 4, sinfo);
 
 	addEdgeLabels(e0);
-	dumb[1].x += del.x;
-	dumb[1].y += del.y;
-	dumb[2].x += del.x;
-	dumb[2].y += del.y;
+	dumb[1] = add_pointf(dumb[1], del);
+	dumb[2] = add_pointf(dumb[2], del);
     }
 }
