@@ -14,6 +14,7 @@
  */
 
 #include <assert.h>
+#include <cgraph/agxbuf.h>
 #include <cgraph/alloc.h>
 #include <common/boxes.h>
 #include <dotgen/dot.h>
@@ -65,7 +66,7 @@ static void adjustregularpath(path *, int, int);
 static Agedge_t *bot_bound(Agedge_t *, int);
 static bool pathscross(Agnode_t *, Agnode_t *, Agedge_t *, Agedge_t *);
 static Agraph_t *cl_bound(graph_t*, Agnode_t *, Agnode_t *);
-static int cl_vninside(Agraph_t *, Agnode_t *);
+static bool cl_vninside(Agraph_t *, Agnode_t *);
 static void completeregularpath(path *, Agedge_t *, Agedge_t *,
 				pathend_t *, pathend_t *, boxf *, int, int);
 static int edgecmp(Agedge_t **, Agedge_t **);
@@ -267,7 +268,7 @@ static void _dot_splines(graph_t * g, int normalize)
     Agedgeinfo_t fwdedgeai, fwdedgebi;
     Agedgepair_t fwdedgea, fwdedgeb;
     edge_t *e, *e0, *e1, *ea, *eb, *le0, *le1, **edges = NULL;
-    path *P = NULL;
+    path P = {0};
     spline_info_t sd;
     int et = EDGE_TYPE(g);
     fwdedgea.out.base.data = (Agrec_t*)&fwdedgeai;
@@ -297,7 +298,6 @@ static void _dot_splines(graph_t * g, int normalize)
 
     mark_lowclusters(g);
     if (routesplinesinit()) return;
-    P = gv_alloc(sizeof(path));
     /* FlatHeight = 2 * GD_nodesep(g); */
     sd.Splinesep = GD_nodesep(g) / 4;
     sd.Multisep = GD_nodesep(g);
@@ -373,7 +373,7 @@ static void _dot_splines(graph_t * g, int normalize)
     qsort(edges, n_edges, sizeof(edges[0]), (qsort_cmpf)edgecmp);
 
     /* FIXME: just how many boxes can there be? */
-    P->boxes = gv_calloc(n_nodes + 20 * 2 * NSUB, sizeof(boxf));
+    P.boxes = gv_calloc(n_nodes + 20 * 2 * NSUB, sizeof(boxf));
     sd.Rank_box = gv_calloc(i, sizeof(boxf));
 
     if (et == EDGETYPE_LINE) {
@@ -423,18 +423,12 @@ static void _dot_splines(graph_t * g, int normalize)
 
 	if (et == EDGETYPE_CURVED) {
 	    int ii;
-	    edge_t* e0;
-	    edge_t** edgelist;
-	    if (cnt == 1)
-		edgelist = &e0;
-	    else
-		edgelist = gv_calloc(cnt, sizeof(edge_t*));
+	    edge_t** edgelist = gv_calloc(cnt, sizeof(edge_t*));
 	    edgelist[0] = getmainedge((edges+ind)[0]);
 	    for (ii = 1; ii < cnt; ii++)
 		edgelist[ii] = (edges+ind)[ii];
 	    makeStraightEdges (g, edgelist, cnt, et, &sinfo);
-	    if (cnt > 1)
-		free (edgelist);
+	    free(edgelist);
 	}
 	else if (agtail(e0) == aghead(e0)) {
 	    int b, r;
@@ -463,10 +457,10 @@ static void _dot_splines(graph_t * g, int normalize)
 	    }
 	}
 	else if (ND_rank(agtail(e0)) == ND_rank(aghead(e0))) {
-	    make_flat_edge(g, &sd, P, edges, ind, cnt, et);
+	    make_flat_edge(g, &sd, &P, edges, ind, cnt, et);
 	}
 	else
-	    make_regular_edge(g, &sd, P, edges, ind, cnt, et);
+	    make_regular_edge(g, &sd, &P, edges, ind, cnt, et);
     }
 
     /* place regular edge labels */
@@ -485,7 +479,7 @@ static void _dot_splines(graph_t * g, int normalize)
 #ifdef ORTHO
 finish :
 #endif
-    /* vladimir: place port labels */
+    /* place port labels */
     /* FIX: head and tail labels are not part of cluster bbox */
     if ((E_headlabel || E_taillabel) && (E_labelangle || E_labeldistance)) {
 	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
@@ -507,7 +501,6 @@ finish :
 	    }
 	}
     }
-    /* end vladimir */
 
 #ifdef ORTHO
     if (et != EDGETYPE_ORTHO && et != EDGETYPE_CURVED)  {
@@ -515,8 +508,7 @@ finish :
     if (et != EDGETYPE_CURVED) {
 #endif
 	free(edges);
-	free(P->boxes);
-	free(P);
+	free(P.boxes);
 	free(sd.Rank_box);
 	routesplinesterm();
     } 
@@ -918,7 +910,6 @@ cleanupCloneGraph (graph_t* g, attr_state_t* attr_state)
     G_ordering = attr_state->G_ordering;
     State = attr_state->State;
 
-    free (attr_state);
     dot_cleanup(g);
     agclose(g);
 }
@@ -932,11 +923,10 @@ static node_t *cloneNode(graph_t *g, node_t *orign) {
     agbindrec(n, "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);
     agcopyattr (orign, n);
     if (shapeOf(orign) == SH_RECORD) {
-        size_t lbllen = strlen(ND_label(orign)->text);
-        char* buf = gv_calloc(lbllen + 3, sizeof(char));
-        sprintf (buf, "{%s}", ND_label(orign)->text);
-	agset (n, "label", buf);
-        free(buf);
+        agxbuf buf = {0};
+        agxbprint(&buf, "{%s}", ND_label(orign)->text);
+        agset (n, "label", agxbuse(&buf));
+        agxbfree(&buf);
     }
 
     return n;
@@ -1099,7 +1089,7 @@ makeSimpleFlatLabels (node_t* tn, node_t* hn, edge_t** edges, int ind, int cnt,
 	poly.pn = 8;
 	poly.ps = (Ppoint_t*)points;
 	pointf *ps = simpleSplineRoute(tp, hp, poly, &pn, et == EDGETYPE_PLINE);
-	if (pn == 0) {
+	if (ps == NULL || pn == 0) {
 	    free(ps);
 	    return;
 	}
@@ -1155,7 +1145,7 @@ makeSimpleFlatLabels (node_t* tn, node_t* hn, edge_t** edges, int ind, int cnt,
 	poly.pn = 8;
 	poly.ps = (Ppoint_t*)points;
 	pointf *ps = simpleSplineRoute(tp, hp, poly, &pn, et == EDGETYPE_PLINE);
-	if (pn == 0) {
+	if (ps == NULL || pn == 0) {
 	    free(ps);
 	    return;
 	}
@@ -1233,7 +1223,6 @@ make_flat_adj_edges(graph_t* g, edge_t** edges, int ind, int cnt, edge_t* e0,
     double midx, midy, leftx, rightx;
     pointf   del;
     edge_t* hvye = NULL;
-    attr_state_t* attrs;
     static int warned;
 
     tn = agtail(e0), hn = aghead(e0);
@@ -1265,18 +1254,17 @@ make_flat_adj_edges(graph_t* g, edge_t** edges, int ind, int cnt, edge_t* e0,
 	return;
     }
 
-    attrs = gv_alloc(sizeof(attr_state_t));
-    auxg = cloneGraph (g, attrs);
+    attr_state_t attrs = {0};
+    auxg = cloneGraph(g, &attrs);
     subg = agsubg (auxg, "xxx",1);
     agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
     agset (subg, "rank", "source");
     rightx = ND_coord(hn).x;
     leftx = ND_coord(tn).x;
     if (GD_flip(g)) {
-        node_t* n;
-        n = tn;
+        node_t *tmp = tn;
         tn = hn;
-        hn = n;
+        hn = tmp;
     }
     auxt = cloneNode(subg, tn);
     auxh = cloneNode(auxg, hn);
@@ -1339,7 +1327,7 @@ make_flat_adj_edges(graph_t* g, edge_t** edges, int ind, int cnt, edge_t* e0,
 
 	e = edges[ind + i];
 	for (; ED_edge_type(e) != NORMAL; e = ED_to_orig(e));
-	auxe = (edge_t*)ED_alg(e);
+	auxe = ED_alg(e);
 	if ((auxe == hvye) & !ED_alg(auxe)) continue; /* pseudo-edge */
 	auxbz = ED_spl(auxe)->list;
 	bz = new_spline(e, auxbz->size);
@@ -1367,7 +1355,7 @@ make_flat_adj_edges(graph_t* g, edge_t** edges, int ind, int cnt, edge_t* e0,
 	}
     }
 
-    cleanupCloneGraph (auxg, attrs);
+    cleanupCloneGraph(auxg, &attrs);
 }
 
 /* makeFlatEnd;
@@ -1488,7 +1476,7 @@ make_flat_labeled_edge(graph_t* g, spline_info_t* sp, path* P, edge_t* e, int et
  */
 static void
 make_flat_bottom_edges(graph_t* g, spline_info_t* sp, path * P, edge_t ** edges, int 
-	ind, int cnt, edge_t* e, int splines)
+	ind, int cnt, edge_t* e, bool use_splines)
 {
     node_t *tn, *hn;
     int j, i, r;
@@ -1545,7 +1533,7 @@ make_flat_bottom_edges(graph_t* g, spline_info_t* sp, path * P, edge_t ** edges,
 
 	pointf *ps = NULL;
 	int pn = 0;
-	if (splines) ps = routesplines(P, &pn);
+	if (use_splines) ps = routesplines(P, &pn);
 	else ps = routepolylines(P, &pn);
 	if (pn == 0) {
 	    free(ps);
@@ -1682,16 +1670,9 @@ make_flat_edge(graph_t* g, spline_info_t* sp, path * P, edge_t ** edges, int ind
     }
 }
 
-/* ccw:
- * Return true if p3 is to left of ray p1->p2
- */
-static int
-leftOf (pointf p1, pointf p2, pointf p3)
-{
-    int d;
-
-    d = (p1.y - p2.y) * (p3.x - p2.x) - (p3.y - p2.y) * (p1.x - p2.x);
-    return d > 0;
+/// Return true if p3 is to left of ray p1->p2
+static bool leftOf(pointf p1, pointf p2, pointf p3) {
+  return (p1.y - p2.y) * (p3.x - p2.x) - (p3.y - p2.y) * (p1.x - p2.x) > 0;
 }
 
 /* makeLineEdge:
@@ -2126,9 +2107,9 @@ static boxf makeregularend(boxf b, int side, double y)
 {
   assert(side == BOTTOM || side == TOP);
   if (side == BOTTOM) {
-    return boxfof(b.LL.x, y, b.UR.x, b.LL.y);
+    return (boxf){{b.LL.x, y}, {b.UR.x, b.LL.y}};
   }
-  return boxfof(b.LL.x, b.UR.y, b.UR.x, y);
+  return (boxf){{b.LL.x, b.UR.y}, {b.UR.x, y}};
 }
 
 #ifndef DONT_WANT_ANY_ENDPOINT_PATH_REFINEMENT
@@ -2234,25 +2215,26 @@ void refineregularends(edge_t *left, edge_t *right, pathend_t *endp, int dir,
  *
  * The second for loop was added by ek long ago, and apparently is intended
  * to guarantee an overlap between adjacent boxes of at least MINW.
- * It doesn't do this, and the ifdef'ed part has the potential of moving 
- * a box within a node for more complex paths.
+ * It doesn't do this.
  */
 static void adjustregularpath(path * P, int fb, int lb)
 {
     boxf *bp1, *bp2;
-    int i, x;
+    int i;
 
     for (i = fb-1; i < lb+1; i++) {
 	bp1 = &P->boxes[i];
 	if ((i - fb) % 2 == 0) {
 	    if (bp1->LL.x >= bp1->UR.x) {
-		x = (bp1->LL.x + bp1->UR.x) / 2;
-		bp1->LL.x = x - HALFMINW, bp1->UR.x = x + HALFMINW;
+		double x = (bp1->LL.x + bp1->UR.x) / 2;
+		bp1->LL.x = x - HALFMINW;
+		bp1->UR.x = x + HALFMINW;
 	    }
 	} else {
 	    if (bp1->LL.x + MINW > bp1->UR.x) {
-		x = (bp1->LL.x + bp1->UR.x) / 2;
-		bp1->LL.x = x - HALFMINW, bp1->UR.x = x + HALFMINW;
+		double x = (bp1->LL.x + bp1->UR.x) / 2;
+		bp1->LL.x = x - HALFMINW;
+		bp1->UR.x = x + HALFMINW;
 	    }
 	}
     }
@@ -2394,10 +2376,9 @@ static edge_t *bot_bound(edge_t * e, int side)
 
 /* common routines */
 
-static int cl_vninside(graph_t * cl, node_t * n)
-{
-    return BETWEEN(GD_bb(cl).LL.x, (double)(ND_coord(n).x), GD_bb(cl).UR.x) &&
-	    BETWEEN(GD_bb(cl).LL.y, (double)(ND_coord(n).y), GD_bb(cl).UR.y);
+static bool cl_vninside(graph_t *cl, node_t *n) {
+  return BETWEEN(GD_bb(cl).LL.x, ND_coord(n).x, GD_bb(cl).UR.x) &&
+         BETWEEN(GD_bb(cl).LL.y, ND_coord(n).y, GD_bb(cl).UR.y);
 }
 
 /* All nodes belong to some cluster, which may be the root graph.
