@@ -9,8 +9,10 @@
  *************************************************************************/
 
 #include "config.h"
+#include <cgraph/agxbuf.h>
 #include <cgraph/alloc.h>
 #include <cgraph/bitarray.h>
+#include <cgraph/list.h>
 #include <sparse/SparseMatrix.h>
 #include <sfdpgen/spring_electrical.h>
 #include <sparse/QuadTree.h>
@@ -48,7 +50,7 @@ spring_electrical_control spring_electrical_control_new(){
   ctrl->step = 0.1;
   ctrl->adaptive_cooling = TRUE;
   ctrl->random_seed = 123;
-  ctrl->beautify_leaves = FALSE;
+  ctrl->beautify_leaves = false;
   ctrl->smoothing = SMOOTHING_NONE;
   ctrl->overlap = 0;
   ctrl->do_shrinking = 1;
@@ -86,7 +88,7 @@ void spring_electrical_control_print(spring_electrical_control ctrl){
   fprintf (stderr, "  Barnes-Hutt constant %.03f tolerance  %.03f maxiter %d\n", ctrl->bh, ctrl->tol, ctrl->maxiter);
   fprintf (stderr, "  cooling %.03f step size  %.03f adaptive %d\n", ctrl->cool, ctrl->step, ctrl->adaptive_cooling);
   fprintf (stderr, "  beautify_leaves %d node weights %d rotation %.03f\n",
-    ctrl->beautify_leaves, 0, ctrl->rotation);
+           (int)ctrl->beautify_leaves, 0, ctrl->rotation);
   fprintf (stderr, "  smoothing %s overlap %d initial_scaling %.03f do_shrinking %d\n",
     smoothings[ctrl->smoothing], ctrl->overlap, ctrl->initial_scaling, ctrl->do_shrinking);
   fprintf (stderr, "  octree scheme %s method %s\n", tschemes[ctrl->tscheme], methods[ctrl->method]);
@@ -305,76 +307,22 @@ static double update_step(int adaptive_cooling, double step, double Fnorm, doubl
 
 #define node_degree(i) (ia[(i)+1] - ia[(i)])
 
-static void check_real_array_size(double **a, int len, int *lenmax){
-  if (len >= *lenmax){
-    *a = gv_recalloc(*a, *lenmax, len + 10, sizeof(double));
-    *lenmax = len + 10;
-  }
-
-}
-static void check_int_array_size(int **a, int len, int *lenmax){
-  if (len >= *lenmax){
-    *a = gv_recalloc(*a, *lenmax, len + 10, sizeof(int));
-    *lenmax = len + 10;
-  }
-
-}
-
-static double get_angle(double *x, int dim, int i, int j){
-  /* between [0, 2Pi)*/
-  int k;
-  double y[2], res;
-  double eps = 0.00001;
-  for (k = 0; k < 2; k++){
-    y[k] = x[j*dim+k] - x[i*dim+k];
-  }
-  if (fabs(y[0]) <= fabs(y[1])*eps){
-    if (y[1] > 0) return 0.5*PI;
-    return 1.5*PI;
-  }
-  res = atan(y[1]/y[0]);
-  if (y[0] > 0){
-    if (y[1] < 0) res = 2*PI+res;
-  } else if (y[0] < 0){
-    res = res + PI;
-  }
-  return res;
-}
-
-static int comp_real(const void *x, const void *y){
-  const double *xx = x;
-  const double *yy = y;
-
-  if (*xx > *yy){
-    return 1;
-  } else if (*xx < *yy){
-    return -1;
-  }
-  return 0;
-}
-static void sort_real(int n, double *a){
-  qsort(a, n, sizeof(double), comp_real);
-}
-
-
 static void set_leaves(double *x, int dim, double dist, double ang, int i, int j){
   x[dim*j] = cos(ang)*dist + x[dim*i];
   x[dim*j+1] = sin(ang)*dist + x[dim*i+1];
 }
 
+DEFINE_LIST(ints, int)
+
 static void beautify_leaves(int dim, SparseMatrix A, double *x){
-  int m = A->m, i, j, *ia = A->ia, *ja = A->ja, k;
+  int m = A->m, i, j, *ia = A->ia, *ja = A->ja;
   int p;
   double dist;
-  int nleaves, nleaves_max = 10;
-  double *angles, maxang, ang1 = 0, ang2 = 0, pad, step;
-  int *leaves, nangles_max = 10, nangles;
+  double step;
 
   assert(!SparseMatrix_has_diagonal(A));
 
   bitarray_t checked = bitarray_new(m);
-  angles = gv_calloc(nangles_max, sizeof(double));
-  leaves = gv_calloc(nleaves_max, sizeof(int));
 
   for (i = 0; i < m; i++){
     if (ia[i+1] - ia[i] != 1) continue;
@@ -382,55 +330,35 @@ static void beautify_leaves(int dim, SparseMatrix A, double *x){
     p = ja[ia[i]];
     if (!bitarray_get(checked, p)) {
       bitarray_set(&checked, p, true);
-      dist = 0; nleaves = 0; nangles = 0;
+      dist = 0;
+      ints_t leaves = {0};
       for (j = ia[p]; j < ia[p+1]; j++){
 	if (node_degree(ja[j]) == 1){
 	  bitarray_set(&checked, ja[j], true);
-	  check_int_array_size(&leaves, nleaves, &nleaves_max);
 	  dist += distance(x, dim, p, ja[j]);
-	  leaves[nleaves] = ja[j];
-	  nleaves++;
-	} else {
-	  check_real_array_size(&angles, nangles, &nangles_max);
-	  angles[nangles++] = get_angle(x, dim, p, ja[j]);
+	  ints_append(&leaves, ja[j]);
 	}
       }
-      assert(nleaves > 0);
-      dist /= nleaves;
-      if (nangles > 0){
-	sort_real(nangles, angles);
-	maxang = 0;
-	for (k = 0; k < nangles - 1; k++){
-	  if (angles[k+1] - angles[k] > maxang){
-	    maxang = angles[k+1] - angles[k];
-	    ang1 = angles[k]; ang2 = angles[k+1];
-	  }
-	}
-	if (2*PI + angles[0] - angles[nangles - 1] > maxang){
-	  maxang = 2*PI + angles[0] - angles[nangles - 1];
-	  ang1 = angles[nangles - 1];
-	  ang2 = 2*PI + angles[0];
-	}
-      } else {
-	ang1 = 0; ang2 = 2*PI; maxang = 2*PI;
-      }
-      pad = MAX(maxang - PI*0.166667*(nleaves-1), 0)*0.5;
-      ang1 += pad*0.95;
-      ang2 -= pad*0.95;
-ang1 = 0; ang2 = 2*PI; maxang = 2*PI;
+      assert(!ints_is_empty(&leaves));
+      dist /= (double)ints_size(&leaves);
+      double ang1 = 0;
+      double ang2 = 2 * M_PI;
+      const double pad = 0.1; // fudge factor to account for the size and
+                              // placement of the nodes themselves
+      ang1 += pad;
+      ang2 -= pad;
       assert(ang2 >= ang1);
       step = 0.;
-      if (nleaves > 1) step = (ang2 - ang1)/(nleaves - 1);
-      for (i = 0; i < nleaves; i++) {
-	set_leaves(x, dim, dist, ang1, p, leaves[i]);
+      if (ints_size(&leaves) > 1) step = (ang2 - ang1) / (double)ints_size(&leaves);
+      for (size_t k = 0; k < ints_size(&leaves); k++) {
+	set_leaves(x, dim, dist, ang1, p, ints_get(&leaves, k));
 	ang1 += step;
       }
+      ints_free(&leaves);
     }
   }
 
   bitarray_reset(&checked);
-  free(angles);
-  free(leaves);
 }
 
 void force_print(FILE *fp, int n, int dim, double *x, double *force){
@@ -692,10 +620,10 @@ static void spring_electrical_embedding_slow(int dim, SparseMatrix A0, spring_el
 #ifdef DEBUG_0
   {
     FILE *f;
-    char fname[10000];
-    strcpy(fname,"/tmp/graph_layout_0_");
-    sprintf(&(fname[strlen(fname)]), "%d",n);
-    f = fopen(fname,"w");
+    agxbuf fname = {0};
+    agxbprint(&fname, "/tmp/graph_layout_0_%d", n);
+    f = fopen(agxbuse(&fname), "w");
+    agxbfree(&fname);
     export_embedding(f, dim, A, x, NULL);
     fclose(f);
   }
@@ -777,10 +705,10 @@ static void spring_electrical_embedding_slow(int dim, SparseMatrix A0, spring_el
 #ifdef DEBUG_PRINT_0
   {
     FILE *f;
-    char fname[10000];
-    strcpy(fname,"/tmp/graph_layout");
-    sprintf(&(fname[strlen(fname)]), "%d",n);
-    f = fopen(fname,"w");
+    agxbuf fname = {0};
+    agxbprint(&fname, "/tmp/graph_layout%d", n);
+    f = fopen(agxbuse(&fname), "w");
+    agxbfree(&fname);
     export_embedding(f, dim, A, x, NULL);
     fclose(f);
   }
@@ -868,10 +796,10 @@ void spring_electrical_embedding(int dim, SparseMatrix A0, spring_electrical_con
 #ifdef DEBUG_0
   {
     FILE *f;
-    char fname[10000];
-    strcpy(fname,"/tmp/graph_layout_0_");
-    sprintf(&(fname[strlen(fname)]), "%d",n);
-    f = fopen(fname,"w");
+    agxbuf fname = {0};
+    agxbprint(&fname, "/tmp/graph_layout_0_%d", n);
+    f = fopen(agxbuse(&fname), "w");
+    agxbfree(&fname);
     export_embedding(f, dim, A, x, NULL);
     fclose(f);
   }
@@ -884,10 +812,11 @@ void spring_electrical_embedding(int dim, SparseMatrix A0, spring_electrical_con
 #ifdef VIS_MULTILEVEL
   {
     FILE *f;
-    char fname[10000];
+    agxbuf fname = {0};
     static int count = 0;
-    sprintf(fname, "/tmp/multilevel_%d",count++);
-    f = fopen(fname,"w");
+    agxbprint(&fname, "/tmp/multilevel_%d", count++);
+    f = fopen(agxbuse(&fname), "w");
+    agxbfree(&fname);
     export_embedding(f, dim, A, x, NULL);
     fclose(f);
   }
@@ -994,10 +923,10 @@ void spring_electrical_embedding(int dim, SparseMatrix A0, spring_electrical_con
 #ifdef DEBUG_PRINT_0
   {
     FILE *f;
-    char fname[10000];
-    strcpy(fname,"/tmp/graph_layout");
-    sprintf(&(fname[strlen(fname)]), "%d",n);
-    f = fopen(fname,"w");
+    agxbuf fname = {0};
+    agxbprint(&fname, "/tmp/graph_layout%d", n);
+    f = fopen(agxbuse(&fname), "w");
+    agxbfree(&fname);
     export_embedding(f, dim, A, x, NULL);
     fclose(f);
   }
@@ -1169,10 +1098,10 @@ static void spring_maxent_embedding(int dim, SparseMatrix A0, SparseMatrix D, sp
 #ifdef DEBUG_0
   {
     FILE *f;
-    char fname[10000];
-    strcpy(fname,"/tmp/graph_layout_0_");
-    sprintf(&(fname[strlen(fname)]), "%d",n);
-    f = fopen(fname,"w");
+    agxbuf fname = {0};
+    agxbprint(&fname, "/tmp/graph_layout_0_%d", n);
+    f = fopen(agxbuse(&fname), "w");
+    agxbfree(&fname);
     export_embedding(f, dim, A, x, NULL);
     fclose(f);
   }
@@ -1356,10 +1285,10 @@ void spring_electrical_spring_embedding(int dim, SparseMatrix A0, SparseMatrix D
 #ifdef DEBUG_0
   {
     FILE *f;
-    char fname[10000];
-    strcpy(fname,"/tmp/graph_layout_0_");
-    sprintf(&(fname[strlen(fname)]), "%d",n);
-    f = fopen(fname,"w");
+    agxbuf fname = {0};
+    agxbprint(&fname, "/tmp/graph_layout_0_%d", n);
+    f = fopen(agxbuse(&fname), "w");
+    agxbfree(&fname);
     export_embedding(f, dim, A, x, NULL);
     fclose(f);
   }
@@ -1459,10 +1388,10 @@ void spring_electrical_spring_embedding(int dim, SparseMatrix A0, SparseMatrix D
 #ifdef DEBUG_PRINT_0
   {
     FILE *f;
-    char fname[10000];
-    strcpy(fname,"/tmp/graph_layout");
-    sprintf(&(fname[strlen(fname)]), "%d",n);
-    f = fopen(fname,"w");
+    agxbuf fname = {0};
+    agxbprint(&fname, "/tmp/graph_layout%d", n);
+    f = fopen(agxbuse(&fname), "w");
+    agxbfree(&fname);
     export_embedding(f, dim, A, x, NULL);
     fclose(f);
   }
@@ -1520,21 +1449,18 @@ void interpolate_coord(int dim, SparseMatrix A, double *x){
 
   free(y);
 }
-static void prolongate(int dim, SparseMatrix A, SparseMatrix P, SparseMatrix R, double *x, double *y, int coarsen_scheme_used, double delta){
+static void prolongate(int dim, SparseMatrix A, SparseMatrix P, SparseMatrix R, double *x, double *y, double delta){
   int nc, *ia, *ja, i, j, k;
   SparseMatrix_multiply_dense(P, x, &y, dim);
 
-  /* xu yao rao dong */
-  if (coarsen_scheme_used > EDGE_BASED_STA && coarsen_scheme_used < EDGE_BASED_STO){
-    interpolate_coord(dim, A, y);
-    nc = R->m;
-    ia = R->ia;
-    ja = R->ja;
-    for (i = 0; i < nc; i++){
-      for (j = ia[i]+1; j < ia[i+1]; j++){
-	for (k = 0; k < dim; k++){
-	  y[ja[j]*dim + k] += delta*(drand() - 0.5);
-	}
+  interpolate_coord(dim, A, y);
+  nc = R->m;
+  ia = R->ia;
+  ja = R->ja;
+  for (i = 0; i < nc; i++){
+    for (j = ia[i]+1; j < ia[i+1]; j++){
+      for (k = 0; k < dim; k++){
+        y[ja[j]*dim + k] += delta*(drand() - 0.5);
       }
     }
   }
@@ -1766,7 +1692,7 @@ static void multilevel_spring_electrical_embedding_core(int dim, SparseMatrix A0
 
 
   Multilevel_control mctrl = NULL;
-  int n, plg, coarsen_scheme_used;
+  int n, plg;
   SparseMatrix A = A0, D = D0, P = NULL;
   Multilevel grid, grid0;
   double *xc = NULL, *xf = NULL;
@@ -1893,25 +1819,19 @@ static void multilevel_spring_electrical_embedding_core(int dim, SparseMatrix A0
       goto RETURN;
     }
     P = grid->P;
-    coarsen_scheme_used = grid->coarsen_scheme_used;
     grid = grid->prev;
     if (Multilevel_is_finest(grid)){
       xf = x;
     } else {
       xf = gv_calloc(grid->n * dim, sizeof(double));
     }
-    prolongate(dim, grid->A, P, grid->R, xc, xf, coarsen_scheme_used, (ctrl->K)*0.001);
+    prolongate(dim, grid->A, P, grid->R, xc, xf, (ctrl->K)*0.001);
     free(xc);
     xc = xf;
     ctrl->random_start = FALSE;
     ctrl->K = ctrl->K * 0.75;
     ctrl->adaptive_cooling = FALSE;
-    if (grid->next->coarsen_scheme_used > VERTEX_BASED_STA &&
-	grid->next->coarsen_scheme_used < VERTEX_BASED_STO){
-      ctrl->step = 1;
-    } else {
-      ctrl->step = .1;
-    }
+    ctrl->step = .1;
   } while (grid);
 
 #ifdef TIME
