@@ -26,7 +26,8 @@ This is somewhat similar to the binary stress model
 
 */
 
-UniformStressSmoother UniformStressSmoother_new(int dim, SparseMatrix A, double *x, double alpha, double M, int *flag){
+static UniformStressSmoother UniformStressSmoother_new(SparseMatrix A,
+                                                       double alpha, double M) {
   UniformStressSmoother sm;
   int i, j, k, m = A->m, *ia = A->ia, *ja = A->ja, *iw, *jw, *id, *jd;
   int nz;
@@ -125,14 +126,12 @@ static SparseMatrix get_distance_matrix(SparseMatrix A, double scaling){
 
 extern void scale_to_box(double xmin, double ymin, double xmax, double ymax, int n, int dim, double *x);
 
-void uniform_stress(int dim, SparseMatrix A, double *x, int *flag){
+void uniform_stress(int dim, SparseMatrix A, double *x) {
   UniformStressSmoother sm;
   double lambda0 = 10.1, M = 100, scaling = 1.;
   int maxit = 300, samepoint = TRUE, i, k, n = A->m;
   SparseMatrix B = NULL;
 
-  *flag = 0;
-
   /* just set random initial for now */
   for (i = 0; i < dim*n; i++) {
     x[i] = M*drand();
@@ -160,19 +159,19 @@ void uniform_stress(int dim, SparseMatrix A, double *x, int *flag){
   B = get_distance_matrix(A, scaling);
   assert(SparseMatrix_is_symmetric(B, false));
 
-  sm = UniformStressSmoother_new(dim, B, x, 1000000*lambda0, M, flag);
+  sm = UniformStressSmoother_new(B, 1000000*lambda0, M);
   UniformStressSmoother_smooth(sm, dim, x, maxit);
   UniformStressSmoother_delete(sm);
 
-  sm = UniformStressSmoother_new(dim, B, x, 10000*lambda0, M, flag);
+  sm = UniformStressSmoother_new(B, 10000*lambda0, M);
   UniformStressSmoother_smooth(sm, dim, x, maxit);
   UniformStressSmoother_delete(sm);
 
-  sm = UniformStressSmoother_new(dim, B, x, 100*lambda0, M, flag);
+  sm = UniformStressSmoother_new(B, 100*lambda0, M);
   UniformStressSmoother_smooth(sm, dim, x, maxit);
   UniformStressSmoother_delete(sm);
 
-  sm = UniformStressSmoother_new(dim, B, x, lambda0, M, flag);
+  sm = UniformStressSmoother_new(B, lambda0, M);
   UniformStressSmoother_smooth(sm, dim, x, maxit);
   UniformStressSmoother_delete(sm);
 
