@@ -10,13 +10,15 @@
 
 #include "config.h"
 #include "gdioctx_wrapper.h"
-
+#include "gdgen_text.h"
+#include "gd_psfontResolve.h"
 #include <stdbool.h>
 #include <stdlib.h>
 #include <stdint.h>
 #include <string.h>
 #include <fcntl.h>
-
+#include <cgraph/alloc.h>
+#include <cgraph/unreachable.h>
 #include <gvc/gvplugin_render.h>
 #include <gvc/gvplugin_device.h>
 #include <gvc/gvcint.h>	/* for gvc->g for agget */
@@ -145,7 +147,7 @@ static void gdgen_end_page(GVJ_t * job)
 {
     gdImagePtr im = job->context;
 
-    gd_context_t gd_context = {{0}};
+    gd_context_t gd_context = {{0}, 0};
 
     gd_context.ctx.putBuf = gvdevice_gd_putBuf;
     gd_context.ctx.putC = gvdevice_gd_putC;
@@ -217,6 +219,8 @@ static void gdgen_end_page(GVJ_t * job)
 
 	case FORMAT_XBM:
 	    break;
+	default:
+	    UNREACHABLE();
 	}
 	gdImageDestroy(im);
 #ifdef MYTRACE
@@ -227,8 +231,7 @@ static void gdgen_end_page(GVJ_t * job)
 }
 
 #ifdef HAVE_GD_FREETYPE
-static void gdgen_missingfont(char *err, char *fontreq)
-{
+static void gdgen_missingfont(char *fontreq) {
     static char *lastmissing = 0;
     static int n_errors = 0;
 
@@ -241,7 +244,7 @@ static void gdgen_missingfont(char *err, char *fontreq)
 	    p = DEFAULT_FONTPATH;
 #endif
 	free(lastmissing);
-	lastmissing = strdup(fontreq);
+	lastmissing = gv_strdup(fontreq);
 	n_errors++;
     }
 }
@@ -287,15 +290,18 @@ void gdgen_text(gdImagePtr im, pointf spf, pointf epf, int fontcolor, double fon
 #ifdef HAVE_GD_FONTCONFIG
         char* fontlist = fontname;
 #else
-        extern char *gd_alternate_fontlist(char *font);
+        extern char *gd_alternate_fontlist(const char *font);
         char* fontlist = gd_alternate_fontlist(fontname);
 #endif
         err = gdImageStringFTEx(im, brect, fontcolor,
                 fontlist, fontsize, fontangle, sp.x, sp.y, str, &strex);
+#ifndef HAVE_GD_FONTCONFIG
+        free(fontlist);
+#endif
 
         if (err) {
             /* revert to builtin fonts */
-            gdgen_missingfont(err, fontname);
+            gdgen_missingfont(fontname);
 #endif
             sp.y += 2;
             if (fontsize <= 8.5) {
@@ -315,8 +321,6 @@ void gdgen_text(gdImagePtr im, pointf spf, pointf epf, int fontcolor, double fon
     }
 }
 
-extern char* gd_psfontResolve (PostscriptAlias* pa);
-
 static void gdgen_textspan(GVJ_t * job, pointf p, textspan_t * span)
 {
     gdImagePtr im = job->context;
@@ -421,9 +425,7 @@ static int gdgen_set_penstyle(GVJ_t * job, gdImagePtr im, gdImagePtr* brush)
 }
 
 static void
-gdgen_bezier(GVJ_t * job, pointf * A, int n, int arrow_at_start,
-	     int arrow_at_end, int filled)
-{
+gdgen_bezier(GVJ_t *job, pointf *A, int n, int filled) {
     obj_state_t *obj = job->obj;
     gdImagePtr im = job->context;
     pointf p0, p1, V[4];
@@ -465,7 +467,7 @@ gdgen_bezier(GVJ_t * job, pointf * A, int n, int arrow_at_start,
 }
 
 static gdPoint *points;
-static int points_allocated;
+static size_t points_allocated;
 
 static void gdgen_polygon(GVJ_t * job, pointf * A, int n, int filled)
 {
@@ -484,9 +486,9 @@ static void gdgen_polygon(GVJ_t * job, pointf * A, int n, int filled)
     fill_ok = filled && obj->fillcolor.u.index != gdImageGetTransparent(im);
 
     if (pen_ok || fill_ok) {
-        if (n > points_allocated) {
-	    points = realloc(points, n * sizeof(gdPoint));
-	    points_allocated = n;
+        if (n > 0 && (size_t)n > points_allocated) {
+	    points = gv_recalloc(points, points_allocated, (size_t)n, sizeof(gdPoint));
+	    points_allocated = (size_t)n;
 	}
         for (i = 0; i < n; i++) {
 	    points[i].x = ROUND(A[i].x);
